#ifndef TRX_H
#define TRX_H
/** \file
   All TRAX definitions.
*/
/*

   $Log: trx.h,v $
   Revision 1.4  2008/02/05 10:55:32  kraemer
   doxygenized

   Revision 1.3  2002/11/25 16:09:28  kraemer
   added new command prototypes

   Revision 1.2  2001/01/15 17:23:36  kraemer
   *** empty log message ***

   Revision 1.1  2001/01/09 14:54:27  kraemer
   *** empty log message ***


*/

#include "u.h"
#include "m.h"

#include "trxtarg.h"
#include "trxxs.h"
#include "trxrnd.h"
#include "trxgeo.h"
#include "trxsrc.h"
#include "trxlmd.h"
#include "trxrun.h"
#if defined(LATER)
#endif
                                      /** \struct STRX
                                          \brief Main structure.
                                      */ 
struct STRX {
   unsigned long ulFlag;              /**< global system status flag */
   struct STRXTARGBASE sTB;           /**< target base */     
   struct STRXXSBASE sXSB;            /**< cross section base */     
   struct STRXRND sRnd;               /**< random numbers */
   struct STRXGEO sGeo;               /**< geometry */
   struct STRXSRC sSrc;               /**< source */
   struct STRXLMD sLmd;               /**< list mode data */
   struct STRXRUN sRun;               /**< run data */
#if defined(LATER)
#endif   
};
                                      /* */
/*
   --- Common status and action flags ----------------------------------------
*/
                                      /* */
                                      /* masks */
#define TRX_MASKGLOBAL 0xFF000000     /* global (e.g. debug) */
#define TRX_MASKFUNC   0x00FF0000     /* function specific flags */
#define TRX_MASKQUAL   0x0000FFFF     /* for additional qualifiers */
                                      /* flags */
#define TRX_DEBUG      0x80000000     /* general check flag */
#define TRX_CHECK      TRX_DEBUG      /* general check flag */
#define TRX_QUIET      0x10000000     /* suppress error messages */
#define TRX_NOMSG      TRX_QUIET

/*
   --- function prototypes
*/
int TRXError( struct SCMDERR *, unsigned long );
int TRXInit ( void *, unsigned long );
int TRXTerm ( void *, unsigned long );

                                      /* command processors */
int TRXCmdTarg(
                const char *, const char *, const char *, const char *, 
                long *, long *, long *, long *, long *,
                long *
              );

int TRXCmdXS(
              const char *, const char *, const char *,
              const char *, const char *, const char *, 
              long *, long *, long *, long *, long *, long *,
              long *, long *, long *, long *, long *,
              long *, long *, long *,
              const char *, const char *, const char *,
              const char *, 
              double *,
              long * 
            );

int TRXCmdRnd(
               long *, long *,
               long *, long *,
               const char *, const char *,
               long *,
               long *,
               long *,
               long *
             );
 
int TRXCmdGeo(
              const char *,
              const char *, const char *, const char *,
              long *, long *, long *, long *, long *, long *,  
              long *  
             );

int TRXCmdSrc(
              const char *,
              const char *, const char *, const char *,
              long *, long *, long *, long *, long *, long *,  
              double *, const char *, const char *,    
              long *  
             );

int TRXCmdLmd( const char *, long *, long *, const char *, const char *,
               long *, 
               long * );

int TRXCmdRun( long *, const char *, long *, long * );

int TRXCmdScore( long * );

int TRXCmdLineTrack( const char *, const char *, 
                     double *, double *, 
                     long * );

int TRXCmdPointTrack( const char *, const char *, long * );

int TRXCmdEvalOut  ( const char *, const char *,
                     double *, double *, long *,
                     double *, double *, long *,
                     long *, long *,
                     long * );

int TRXCmdEvalTESpc( const char *, const char *,
                     double *, double *, long *,
                     long *, long *,
                     double *, double *, long *,
                     long *, long *,
                     long * );

int TRXCmdEvalRDose( const char *, const char *, 
                     double *,
                     double *, double *, 
                     double *, double *, long *,
                     long *, long *,
                     long * );

int TRXCmdEvalADose( const char *, const char *, 
                     double *, double *, 
                     double *, double *, long *,
                     long * );

int TRXCmdEvalOPAC ( const char *, const char *, 
                     double *,
                     double *, double *, 
                     double *, double *, long *,
                     long *, long *,
                     long * );

int TRXCmdEvalrzDose( const char *, const char *, 
                      double *,
                      double *, double *, long *, long *, long *, const char *,
                      double *, double *, long *, long *, long *, const char *,
                      long * );

int TRXCmdEvalSphereDose( const char *, const char *, 
                          double *,
                          const char *, 
                          double *, double *, long *,
                          long *, long *,
                          const char *, 
                          long * );

int TRXCmdEvalMDose( const char *, const char *, 
                     double *, double *, 
                     double *, double *, long *,
                     long *, long *,
                     double *, const char *,
                     long *,
                     long *,
		     long * );

int TRXCmdEvalCrossers( const char *, const char *,
                        const char *,   
		        long * );

int TRXCmdEvalUsr  ( const char *, const char *, const char *, const char *,
                     long * );

int TRXCmdEvalVolESpc( const char *, const char *,
                        double *, double *, long *,
                        long *, long *,
		        long * );
                                      /*  */
int TRXCmdEqui( const char *, const char *, double *,
                long * );
            
int TRXCmdProjTest    ( const char *, long * );
int TRXCmdTargTest    ( const char *, long * );
int TRXCmdXSTest      ( const char *, long * );
int TRXCmdEMFldTest   ( const char *, long * );
int TRXCmdRndTest     ( const char *, long * );
int TRXCmdVolTest     ( const char *, long * );
int TRXCmdGeoTest     ( const char *, long * );
int TRXCmdParticleTest( const char *, long * );
int TRXCmdSrcTest     ( const char *, long * );
int TRXCmdRunTest     ( const char *, long * );
int TRXCmdEvalTest    ( const char *, long * );
int TRXCmdLmdTest     ( const char *, long * );

int TRXCmdTest        ( const char *, long * );

/*
   --- error handling
*/
enum { ERRTRX_SYS=1,
       ERRTRX_BADADDR,
       ERRTRX_CMDFAILED,
       ERRTRX_CMDPARNOTYET,
       ERRTRX_NOFILE,
       ERRTRX_BADNUMBER,
       ERRTRX_NOXSFORMATERIAL,
       ERRTRX_FILEEMPTY,
   ERRTRX_LOADFAILED,
   ERRTRX_BINDFAILED
};

#endif
