#ifndef TRXXS_H
#define TRXXS_H
/*

   Definitions for cross section tables

   Copyright  1992-2002 M.Kraemer, GSI Darmstadt

   $Log: trxxs.h,v $
   Revision 1.2  2002/11/25 16:39:38  kraemer
   locate macros; outside and emfld pseudo-interactions

   Revision 1.1  2001/01/09 14:54:44  kraemer
   *** empty log message ***


*/

#include <limits.h>

#include "m.h"
#include "cmderr.h"
#include "trxmmap.h"
#include "trxproj.h"
#include "trxtarg.h"

                                      /* Barkas formula */
//#define BARKAS(Z0,beta2) (Z0) * ( ( 1E0 - exp( -125E0 * sqrt(beta2) * pow( (Z0), -2E0/3E0 ) ) ) )
                                      /* Common status and action flags */
                                      /* */
                                      /* masks */
#define TRXXS_MASKGLOBAL 0xFF000000   /* global (e.g. debug) */
#define TRXXS_MASKFUNC   0x00FF0000   /* function specific flags */
#define TRXXS_MASKQUAL   0x0000FFFF   /* for additional qualifiers */
                                      /* global flags */
#define TRXXS_DEBUG      0x80000000   /* general check flag */
#define TRXXS_QUIET      0x40000000   /* suppress error messages */
#define TRXXS_NOMSG      TRXXS_QUIET

struct STRXXSCALC {
   double dRelChange;
   double dvELim[2];
   double dvWLim[2];
   double dvALim[2];                  /* cos(th) */
   int nE, nW, nA;
   const char *pcMethod;  
};    

/*

   lIA = TRXXSIActFromStr : ( pcIA, ulFlag )
                            x to test
                            o determine interaction type from character code
                            o calls:                            

*/

long TRXXSIActFromStr( const char *, unsigned long );

                                      /* Cross sections for a single shell */
struct STRXXSSH {
   double dSigTot;                    /* total cross section this shell */
   double dSigTotScaled;              /* scaled for interpolation */
   double dLET,
          dEkin,                      /* mean kinetic energy */
          dEbind;                     /* binding energy this shell [keV] */
   double dvW[4];                     /* aux. for ioniz. secondary energy */
   int  nW, nA;                       /* in: # of secondary energy/angle values */          
                                     
   double **pdSig2;                   /* [nW][nA] */       
   double **pdCum2;                   /* [nW][nA] cumulated, normalized */
   double  *pdSigW;                   /* [nW] */
   double  *pdCumW;                   /* [nW] cumulated, normalized */
   double  *pdSigA;                   /* [nA] */
   double  *pdCumA;                   /* [nA] cumulated, normalized */
   double  *pdW;                      /* [nW] secondaries energy scale
                                         ions: W/Wbe   
                                      */
   double  *pdA;                      /* [nA] angular scale */    
   unsigned long ulSpace;             /* space counter */
};

/* 
   --- Cross sections for one primary energy 
*/
struct STRXXSE {
   double dE;                         /* primary Energy,
                                         ions: MeV/u
                                         else: keV
                                      */
   double dEScaled;                   /* scaled for interpolation */
   double dSigTot;                    /* total cross section [cm**2] */
   double dSigTotScaled;              /* scaled for interpolation */
   double dkm;                        /* mean potential energy transfer */
   double dEkin;                      /* mean kinetic energy of ejected particles */                
   double dvW[4];                     /* aux. for ioniz. secondary energy */
   double dbeta2;
   double dgamma;  
   double dWNorm;                     /* scale factor for secondary energies
                                         ions: binary peak 
                                      */
   long lMethod;                      /* calc. method */
   int  nSh;                          /* # of shells */
   struct STRXXSSH *psSh;             /* [nSh+1] shell specific stuff
                                         [0..nSh-1] single shells
                                         [nSh]      all shells
                                      */
   unsigned long ulSpace;             /* space counter */
};

                                      /* common macro to find primary energy bin */
#define TRXXSLOCATEE(iE,psXSELo,psXSEHi,iLo,iHi,psXS,dEScaled)\
   iLo = 0; iHi = (psXS)->nE-1; iE = iLo;\
   for(;;) {\
      if ( ( iE = ( ((iLo)+(iHi)) >> 1 ) ) == iLo ) break;\
      if ( (dEScaled) == (psXS)->psXSE[iE].dEScaled ) break;\
      if ( (dEScaled) <  (psXS)->psXSE[iE].dEScaled ) { iHi = iE; }\
      else                                            { iLo = iE; }\
   }\
   psXSELo = (psXS)->psXSE + (iE);\
   (iE)++; iE = min( (iE), ((psXS)->nE-1) );\
   psXSEHi = (psXS)->psXSE + (iE);
                                  
                                      /* common macro to find cos(angle) */
#define TRXXSLOCATEA(iA,dCos,psXSShLo,psXSShHi,iLo,iHi,ddLo,ddHi,psXS,ddEScaled,dq,dd)\
   iLo = 0; iHi = (psXSShLo)->nA-1; iA = iLo;\
   for(;;) {\
      if ( ( iA = ( ((iLo)+(iHi)) >> 1 ) ) == iLo ) break;\
      dd = ( (psXSShHi) -> pdCumA[iA] - (psXSShLo) -> pdCumA[iA] ) * (ddEScaled) +\
                                        (psXSShLo) -> pdCumA[iA];\
      if ( (dq) == (dd) ) break;\
      if ( (dq)  < (dd) ) { iHi = iA; }\
      else                { iLo = iA; }\
   }\
   ddHi = ( (psXSShHi) -> pdCumA[(iA)+1] - (psXSShLo) -> pdCumA[(iA)+1] ) * (ddEScaled) +\
                                         (psXSShLo) -> pdCumA[(iA)+1];\
   ddLo = ( (psXSShHi) -> pdCumA[iA  ] - (psXSShLo) -> pdCumA[iA    ] ) * (ddEScaled) +\
                                       (psXSShLo) -> pdCumA[iA    ];\
   dCos = ( (psXSShLo) -> pdA[iA+1] - (psXSShLo) -> pdA[iA  ] )\
        / ( (ddHi) - (ddLo) )\
        * ( (dq  ) - (ddLo) )\
        + (psXSShLo) -> pdA[iA  ];                                  
                                      /* common macro to find secondary energies */
#define TRXXSLOCATEW(iW,dW,psXSShLo,psXSShHi,iLo,iHi,ddLo,ddHi,psXS,ddEScaled,dq,dd)\
   iLo = 0; iHi = (psXSShLo)->nW-1; iW = iLo;\
   for(;;) {\
      if ( ( iW = ( ((iLo)+(iHi)) >> 1 ) ) == iLo ) break;\
      dd = ( (psXSShHi) -> pdCumW[iW] - (psXSShLo) -> pdCumW[iW] ) * (ddEScaled) +\
                                        (psXSShLo) -> pdCumW[iW];\
      if ( (dq) == (dd) ) break;\
      if ( (dq)  < (dd) ) { iHi = iW; }\
      else                { iLo = iW; }\
   }\
   ddHi = ( (psXSShHi) -> pdCumW[(iW)+1] - (psXSShLo) -> pdCumW[(iW)+1] ) * (ddEScaled) +\
                                           (psXSShLo) -> pdCumW[(iW)+1];\
   ddLo = ( (psXSShHi) -> pdCumW[iW  ] - (psXSShLo) -> pdCumW[iW    ] ) * (ddEScaled) +\
                                         (psXSShLo) -> pdCumW[iW    ];\
   dW   = ( (psXSShLo) -> pdW[iW+1] - (psXSShLo) -> pdW[iW  ] )\
        / ( (ddHi) - (ddLo) )\
        * ( (dq  ) - (ddLo) )\
        + (psXSShLo) -> pdW[iW  ];                                  
                                      /* calc. methods */
enum { TRXXS_METHNONE=0, 
       TRXXS_METHBEA, TRXXS_METHRUDD, 
       TRXXS_METHHKS, TRXXS_METHCDWEIS, 
       TRXXS_METHREL,
       TRXXS_METHRBEB,
       TRXXS_METHBEB,
       TRXXS_METHEMP,
       TRXXS_METHRFORD,
     };
                                      /* function prototypes */

/*
   --- data base for one projectile/target/interaction combination
*/

struct STRXXS {
   unsigned long ulFlag;              /* */   
            long  lIA;                /* interaction type */
   long lMethod;                      /* calc. method */
   int nE;                            /* # of primary energies */
   struct STRXXSE  *psXSE;            /* [nE] table of cross sections */
   struct STRXPROJ sP;                /* specifies projectile properties */
   struct STRXTARG sT;                /* specifies target properties */
   struct STRXMMAP sM;
   struct SCMDERR sCE;
   unsigned long ulSpace;             /* space counter */
};
                                      /* enums */
                                      /* interaction types */     
enum {
                                      /* true interactions (table based) */ 
   TRXXS_NONE=0,
   TRXXS_BOUNDARY,
   TRXXS_CUTOFF,
   TRXXS_ELASTIC,                     /* 3 */    
   TRXXS_IONIZATION, 
   TRXXS_EXCITATION,
   TRXXS_BREMS, 
   TRXXS_ANNIH,
   MTRXXS,
                                      /* pseudo interactions */ 
                                      /* codes must be < 32 ! */
   TRXXS_OUTSIDE=31,
   TRXXS_EMFLD=TRXXS_OUTSIDE-1,
   TRXXS_MAXPSEUDO,
   TRXXS_BOR=LONG_MAX-1, TRXXS_EOR    /* dummies for begin/end-of-run */
};
enum {
   TRXXS_PHOTO=TRXXS_IONIZATION,      /* 4 */ 
   TRXXS_COMPTON=TRXXS_EXCITATION,    /* 5 */
   TRXXS_PAIR=TRXXS_ANNIH,            /* 7 */ 
};
                                      /* */
#define TRXXS_REFCOPY    0x00000004   /* do not free() */
#define TRXXS_VALID      0x00000001   /* marks valid cross section */  
#define TRXXS_ENABLE     0x00000002   /* marks enabled cross section */  
#define TRXXS_SWAP       0x00010000


#define TRXXS_CHARS "???lixbafcp"

enum {
   TRXXS_BITBOUNDARY   = ( 1L << TRXXS_BOUNDARY ), 
   TRXXS_BITOUTSIDE    = ( 1L << TRXXS_OUTSIDE ), 
   TRXXS_BITCUTOFF     = ( 1L << TRXXS_CUTOFF ), 
   TRXXS_BITELASTIC    = ( 1L << TRXXS_ELASTIC ), 
   TRXXS_BITIONIZATION = ( 1L << TRXXS_IONIZATION ), 
   TRXXS_BITEXCITATION = ( 1L << TRXXS_EXCITATION ), 
   TRXXS_BITBREMS      = ( 1L << TRXXS_BREMS ), 
   TRXXS_BITANNIH      = ( 1L << TRXXS_ANNIH ), 
   TRXXS_BITEMFLD      = ( 1L << TRXXS_EMFLD ), 
};
enum {
   TRXXS_BITPHOTO      = ( 1L << TRXXS_PHOTO ), 
   TRXXS_BITCOMPTON    = ( 1L << TRXXS_COMPTON ), 
   TRXXS_BITPAIR       = ( 1L << TRXXS_PAIR ), 
};
/*


   Functions to handle a single cross section
   ------------------------------------------

   TRXXSCalc     : ( psXS, ulIA, &(psXSP->sP), psT, ulFlag )
                12 o todo
                   o calls:
                     o TRXXSCalcEL
                     o TRXXSCalcEX
                     o TRXXSCalcEI
                     o TRXXSCalcEB
                     o TRXXSCalcII   
                     o TRXXSCalcIX
                     o TRXXSCalcXP
                     o TRXXSCalcXC
                       ! CumA < 0, >1,
                     o TRXXSCalcXF
                     o TRXXSCalcXL
                     o TRXXSCalcPL
                     o TRXXSCalcPX
                     o TRXXSCalcPI
                     o TRXXSCalcPB
                     o TRXXSCalcPA
                                            
   TRXXSCopy     : ( psXSPFr->sXS+ulIA, psXSPTo->sXS+ulIA, ulFlag )
                 6 x to test
                   x reference copy only
                   o how to support memory mapping ?
                   o calls:
                   
   TRXXSDisable  : ( psXS, ulFlag )
                 8 x to test
                   o calls:

   TRXXSDup      : ( psXSPFr->sXS+ulIA, psXSPTo->sXS+ulIA, ulFlag )
                 7 x to test
                   x real copy w/ allocation
                   o how to support memory mapping ?
                   o calls:
                     o TRXXSFree()
                     o TRXXSInit()
                     
   TRXXSEnable   : ( psXS, ulFlag )
                 9 x to test
                   o calls:

   TRXXSExportgd : ( psXS, pcPrefix, ulFlag )
                11 x to test
                   x filenames like for /write, append ".gd" 
		   o add some xsect specific stuff (scaling)
		   o colours
		     o i: red
		     o x: green
		     o l: blue
		     o b:
		     o a:                        
                   o calls:
		     o TRXXSIsValid()

   TRXXSFree     : ( &sXS, ulFlag )
                 3 x to test
		   o account for mmap()ed files
                   o calls:
		     TRXXSInit() 
                   
   TRXXSInit     : ( &sXS, ulFlag )   
                 2 x to test
		   o account for mmap()ed files
                   o calls:
                     o TRXProjInit
                     o TRXTargInit
                   
   TRXXSInspect  : ( psXS, ulFlag )
                 1 x to test
                   o calls:
                     o TRXProjInspect
                     o TRXTargInspect
                   
   TRXXSIsValid  : ( psXS, ulFlag ) 
                 4 x to test
                   o tests:
                     x nE > 0; 
                     x sigma >= 0  
                     x 0 < cum < 1;
                     x monotonic 
                   o calls:
                     o TRXProjIsValid
                     o TRXTargIsValid

   TRXXSList     :                   
                10 x to test
                   x only non-array stuff
                   o calls:
                     o TRXXSpace
                      
   TRXXSSpace    : count space
                 5 x to test
                   o exact numbers still to check
                   o calls
                    
   TRXXSRead     : ( &sXS, pcFile, ulFlag )
                14 o todo
                   o memory mapping
                   o calls: 
                   
   TRXXSWrite    : ( &sXS, ulFlag )
                13 o todo   
                   ? memory mapping
                   o format IFF-like
                   o file name conventions:
                     o ions:
                              [<massnumber>]<projectile>[<charge>].<target>.i
                              [<massnumber>]<projectile>[<charge>].<target>.x
                     o e-:
                              e-.<target>.i
			                  ^ problem ?
                              e-.<target>.x
                              e-.<target>.l
                              e-.<target>.<solidstatestuff>
                              e-.<target>.b
                     o e+:
                              e+.<target>.i
			                  ^ problem ?
                              e+.<target>.x
                              e+.<target>.l
                              e+.<target>.<solidstatestuff>
                              e+.<target>.a
			                  ^ problem ?
                              e+.<target>.b
                     o x:
                              x.<target>.f
			                 ^ problem ?
                              x.<target>.c
			                 ^ problem ?
                              x.<target>.p
                              x.<target>.l
                   o calls:    


*/

                                      /* function prototypes */
int TRXXSCalcXC( struct STRXXS *,
                 struct STRXPROJ *, struct STRXTARG *,
                 struct STRXXSCALC *,
                 unsigned long
               );
int TRXXSCalcXF( struct STRXXS *,
                 struct STRXPROJ *, struct STRXTARG *,
                 struct STRXXSCALC *,
                 unsigned long
               );

int TRXXSCalcII( struct STRXXS *,
                 struct STRXPROJ *, struct STRXTARG *,
                 struct STRXXSCALC *,
                 unsigned long
               );

int TRXXSCalcIX( struct STRXXS *,
                 struct STRXPROJ *, struct STRXTARG *,
                 struct STRXXSCALC *,
                 unsigned long
               );

int TRXXSCalcEI( struct STRXXS *,
                 struct STRXPROJ *, struct STRXTARG *,
                 struct STRXXSCALC *,
                 unsigned long
               );

int TRXXSCalcEL( struct STRXXS *,
                 struct STRXPROJ *, struct STRXTARG *,
                 struct STRXXSCALC *,
                 unsigned long
               );

int TRXXSCalcEX( struct STRXXS *,
                 struct STRXPROJ *, struct STRXTARG *,
                 struct STRXXSCALC *,
                 unsigned long
               );


int TRXXSCalc( struct STRXXS *, unsigned long, 
               struct STRXPROJ *, struct STRXTARG *,
               struct STRXXSCALC *,
               unsigned long
             );

int TRXXSCopy    ( struct STRXXS *, struct STRXXS *, unsigned long );
int TRXXSDisable ( struct STRXXS *, unsigned long );
int TRXXSDup     ( struct STRXXS *, struct STRXXS *, unsigned long );
int TRXXSEnable  ( struct STRXXS *, unsigned long );
int TRXXSExportgd( struct STRXXS *, const char *, unsigned long );
int TRXXSFree    ( struct STRXXS *, unsigned long );
int TRXXSInit    ( struct STRXXS *, unsigned long );
int TRXXSInspect ( struct STRXXS *, unsigned long );
int TRXXSIsValid ( struct STRXXS *, unsigned long );
int TRXXSList    ( struct STRXXS *, unsigned long );
int TRXXSMerge   ( struct STRXXS *, struct STRXXS *, unsigned long );
int TRXXSRead    ( struct STRXXS *, const char *, unsigned long );
int TRXXSSpace   ( struct STRXXS *, unsigned long );
int TRXXSWrite   ( struct STRXXS *, const char *, unsigned long );

struct STRXXS *_TRXXSTest( unsigned long );             
/*
   --- data base for one projectile
*/

struct STRXXSPROJ {
   unsigned long ulSpace;             /* space counter */
   unsigned long ulPad;               /* */
   double dECutoff;                   /* cutoff energy, from dissipative xs */
   struct STRXPROJ sP;                /* specifies projectile properties */
   struct STRXXS sXS[MTRXXS];         /* */
   struct SCMDERR sCE;
};
/*

   Functions to handle cross sections for one projectile
   -------------------------------------------------

   TRXXSProjCalc     : ( xsp, target, ia, calc, flag )
                    13 x to test
                       o calls:
                         o TRXXSCalc( psXS, ulIA, &(psXSP->sP), psT, ulFlag )
                         o TRXXSIActFromStr
                       
   TRXXSProjCopy     : ( xspto, xspfr, flag )
                     8 x to test
                       o copy valid ia
                       o memory mapping ?
                       o calls:
                         o TRXXSCopy( psXSPTo->sXS+ulIA, psXSPFr->sXS+ulIA, ulFlag ) 
                         o TRXProjCopy()
			   
   TRXXSProjDelete   : ( psXSP, pcIA, ulFlag )
                    10 x to test
                       o calls:
                         o TRXXSFree
                         o TRXXSIActFromStr
                         
   TRXXSProjDisable  : ( xsproj, ia, flag )
                   11a x to test
                       o wildcards for <ia>
                       o calls:
                         o TRXXSDisable( psXS, ulFlag )
                         o TRXXSIActFromStr
                         
   TRXXSProjDup      : ( xsprojto, xsprojfr, flag )
                     9 x to test
                       o calls:
                         o TRXProjDup
                         o TRXXSDup()
                         
   TRXXSProjEnable   : ( psXST->sXSP+iP, pcIA, ulFlag )
                   11b x to test
                       o wildcards for <ia>
                       o calls:
                         o TRXXSEnable( psXS, ulFlag )
                         o TRXXSIActFromStr
                            
   TRXXSProjExportgd : ( xsproj, prefix, ia, flag )
                     7 x to test
                       o calls:
		         o TRXXSIsValid()
                         o TRXXSExportgd( psXS, pcPrefix, ulFlag )
                         o TRXXSIActFromStr
                         
   TRXXSProjFree     : ( xsproj, ulFlag )
                     3 x to test
		       o consider ref copy ?
                       o calls:
                         o TRXXSFree
                         o TRXProjFree
                         o TRXXSProjInit();
                         
   TRXXSProjInit     : ( xsproj )
                     2 o to test
                       o calls:
                         o TRXProjInit
                         o TRXXSInit
                       
   TRXXSProjInspect  : ( xsproj, ia, flag )
                     1 x to test
                       o calls:
                         o TRXProjInspect
                         o TRXXSIActFromStr
                         o TRXXSInspect( psXS, ulFlag )
                       
   TRXXSProjIsValid  : ( xsproj, ia, flag )
                     4 x to test
                       o calls:
                         o TRXXSIActFromStr
                         o TRXXSIsValid( psXS, ulFlag )
                         o TRXXSProjIsValid( psXS, ulFlag )
                       
   TRXXSProjList     : ( xsproj, ia, flag )
                     6 x to test
                       o calls:
                         o TRXXSIActFromStr
                         o TRXXSList( psXS, ulFlag )
                         o TRXXSIsValid( psXS, ulFlag )
			 o TRXProjSpace()
                       
   TRXXSProjMerge    : ( xsprojto, xsprojfr, flag )
                    12 x to test
                       o calls:
                         o TRXProjIsValid
                         o TRXProjDup
                         o TRXProjMatch
                         o TRXProjFree
                         o TRXXSIsValid()
                         o TRXXSDup()
                         o TRXXSFree()
                         

   TRXXSProjSpace    : count space
                     5 x to test
		       o exact numbers still to check
                       o calls
                    
   TRXXSProjWrite    : ( xsproj, prefix, ia, flag )
                       o to complete/test
                       o wildcards for <ia>
                          o format IFF-like
                          o file name conventions:
                            o ions:
                              [<massnumber>]<projectile>[<charge>].<target>.i
                              [<massnumber>]<projectile>[<charge>].<target>.x
                            o e-:
                              e-.<target>.i
                              e-.<target>.x
                              e-.<target>.l
                              e-.<target>.<solidstatestuff>
                              e-.<target>.b
                            o e+:
                              e+.<target>.i
                              e+.<target>.x
                              e+.<target>.l
                              e+.<target>.<solidstatestuff>
                              e+.<target>.a
                              e+.<target>.b
                            o x:
                              x.<target>.f
                              x.<target>.c
                              x.<target>.p
                              x.<target>.l
                       o calls:
                         o TRXXSIActFromStr
                         o TRXXSWrite( psXS, ulFlag )
                       
*/
                                      /* function prototypes */
int TRXXSProjCalc( struct STRXXSPROJ *,  
                   struct STRXTARG *,
		   const char *,
                   struct STRXXSCALC *,
                   unsigned long
                 );
int TRXXSProjCopy    ( struct STRXXSPROJ *, struct STRXXSPROJ *, unsigned long );
int TRXXSProjDelete  ( struct STRXXSPROJ *, const char *, unsigned long );
int TRXXSProjDisable ( struct STRXXSPROJ *, const char *, unsigned long );
int TRXXSProjDup     ( struct STRXXSPROJ *, struct STRXXSPROJ *, unsigned long );
int TRXXSProjEnable  ( struct STRXXSPROJ *, const char *, unsigned long );
int TRXXSProjExportgd( struct STRXXSPROJ *, const char *, const char *, unsigned long );
int TRXXSProjFree    ( struct STRXXSPROJ *, unsigned long );
int TRXXSProjInit    ( struct STRXXSPROJ *, unsigned long );
int TRXXSProjInspect ( struct STRXXSPROJ *, const char *, unsigned long );
int TRXXSProjIsValid ( struct STRXXSPROJ *, const char *, unsigned long );
int TRXXSProjList    ( struct STRXXSPROJ *, const char *, unsigned long );
int TRXXSProjMerge   ( struct STRXXSPROJ *, struct STRXXSPROJ *, unsigned long );
int TRXXSProjSpace   ( struct STRXXSPROJ *, unsigned long );
int TRXXSProjWrite   ( struct STRXXSPROJ *, const char *, const char *, unsigned long );

struct STRXXSPROJ *_TRXXSProjTest( unsigned long );             


/*
   --- data base for one material/target
*/

struct STRXXSTARG {
   struct STRXTARG sT;                /* specifies target properties */
   unsigned long ulFlag;
   unsigned long ulSpace;             /* space counter */
   struct STRXXSPROJ sXSP[MTRXPROJ];  /* */
   struct SCMDERR sCE;
};
/*
   Functions to handle cross section base
   --------------------------------------

   TRXXSTargCalc:    ( &sXST, pcProj, pcIA, psXSC, ulFlag );
                  13 x to test
		     ? wildcards for projectile 
		     o calls:
                       o TRXProjInit( &sP, ulFlagProj );
                       o TRXProjFromStr( &sP, pcProj, ulFlagProj );
                       o TRXProjDup( &(psXSP->sP), &sP, ulFlagProj );
                       o TRXProjFree( &sP, ulFlagProj );
                       o TRXXSProjCalc(); 
		      
   TRXXSTargCopy:    ( psXSTTo, psXSTFr, ulFlag )
                   8 x to test
		     o calls:
		       o TRXXSProjCopy
		       
   TRXXSTargDelete:  ( psXST, pcProj, pcIA, ulFlag )
                  10 x to test
		     o calls:
                       o TRXProjInit( &sP, ulFlagProj );
		       o TRXProjFree( &sP, ulFlagProj );
		       o TRXProjFromStr( &sP, pcProj, ulFlagProj );
		       o TRXXSProjDelete( psXSP, pcIA, ulFlag );
		              
   TRXXSTargDisable: ( psXST, pcProj, pcIA, ulFlag )
   		 11a x to test
		     o calls
                       o TRXProjInit( &sP, ulFlagProj );
		       o TRXProjFree( &sP, ulFlagProj );
		       o TRXProjFromStr( &sP, pcProj, ulFlagProj );
		       o TRXXSProjDisable( psXSP, pcIA, ulFlag );
		      
   TRXXSTargDup:     ( psXSTTo, psXSTFr, ulFlag )
                   9 x to test
		     o calls:
		       o TRXTargDup()
                       o TRXXSTargInit( psXSTTo, ulFlag );
                       o TRXXSTargFree( psXSTTo, ulFlag );
                       o TRXXSProjDup( psXSTTo->sXSP+iP, psXSTFr->sXSP+iP, ulFlag );
		      
   TRXXSTargEnable:  ( psXST, pcProj, pcIA, ulFlag )
                 11b x to test
		     o calls 
                       o TRXProjInit( &sP, ulFlagProj );
		       o TRXProjFree( &sP, ulFlagProj );
		       o TRXProjFromStr( &sP, pcProj, ulFlagProj );
		       o TRXXSProjEnable( psXSP, pcIA, ulFlag );
		     
   TRXXSTargExportgd:( psXST, pcPrefix, pcProj, pcIA, ulFlag )
                   7 x to test
		     o wildcards for proj, ia
		     o calls:
                       o TRXProjInit( &sP, ulFlagProj );
		       o TRXProjFree( &sP, ulFlagProj );
		       o TRXProjFromStr( &sP, pcProj, ulFlagProj );
		       o TRXXSProjExportgd( psXSP, pcIA, ulFlag );
		       
   TRXXSTargFree:    ( psXST, ulFlag )
                   3 o todo
		     o consider refcopy
		     o calls:
                       o TRXTargFree( &(psXST -> sT), ulFlagTarg );                                   
                       o TRXXSProjFree( psXST->sXSP+iP, ulFlag );
                       o TRXXSTargInit();                                   
		     
   TRXXSTargInit   : ( pxXST, flag )
                   2 x to test
		     o calls:
                       o TRXTargInit( &(psXST -> sT), ulFlagTarg );                                   
                       o TRXXSProjInit( psXST->sXSP+iP, ulFlag );
		     
   TRXXSTargInspect: ( psXST, pcProj, pcIA, ulFlag )
                   1 x to test
		     o calls:
		       o TRXTargInspect()
                       o TRXProjInit( &sP, ulFlagProj );
                       o TRXProjFromStr( &sP, pcProj, ulFlagProj );
                       o TRXProjFree( &sP, ulFlagProj );
                       o TRXXSProjInspect();

   TRXXSTargIsValid: ( psXST, ulFlag)
                   4 x to test
		     o calls:
		       o TRXXSProjIsValid( psXST->sXSP+iP, pcIA, ulFlag )
		       
   TRXXSTargList   : ( psXST, pcProj, pcIA, ulFlag )	
                   6 x to test
		     o improve layout		     	     
		     o calls:
                       o TRXProjInit( &sP, ulFlagProj );
		       o TRXProjFree( &sP, ulFlagProj );
		       o TRXProjFromStr( &sP, pcProj, ulFlagProj );
		       o TRXXSProjList( psXSP, pcIA, ulFlag );
		       o TRXTargSpace()
                       
   TRXXSTargMerge  : ( psXSTTo, psXSTFr, ulFlag )
                     duplicate only valid entries fr => to
                  12 x to test
		     o calls:
		       o TRXTargIsValid()
		       o TRXTargFree()
		       o TRXTargDup()
		       o TRXTargMatch()
		       o TRXXSProjMerge()
		       
   TRXXSTargRead:    ( &sXST, full-file-name, ulFlag )
                     o todo
		     o inspect extension for valid ia
		     o proj, targ from contents		     
                      
		     o calls:
                       o TRXXSRead( &sXS, pcFile, ulFlag );
                       o TRXXSInit( &sXS, ulFlag );
                       o TRXXSFree( &sXS, ulFlag );
		     
   TRXXSTargSpace  : count space
                   5 x to test
		     o exact numbers still to verify
                     o calls
                       o TRXTargSpace()
		       o TRXXSProjSpace()
                    
   TRXXSTargWrite  : ( psXST, pcPrefix, pcProj, pcIA, ulFlag )
                     o todo
		     o consider TRXXS_SWAP
		     o calls:
                       o TRXProjInit( &sP, ulFlagProj );
                       o TRXXSProjWrite( psXST->sXSP+iP, pcPrefix, pcIA, ulFlag );
                       o TRXProjFromStr( &sP, pcProj, ulFlagProj );
                       o TRXProjFree( &sP, ulFlagProj );
		     
*/
                                      /* function prototypes */
int TRXXSTargCalc( struct STRXXSTARG *,  
                   const char *,
		   const char *,
                   struct STRXXSCALC *,
                   unsigned long
                 );
int TRXXSTargCopy    ( struct STRXXSTARG *, struct STRXXSTARG *, unsigned long );
int TRXXSTargDelete  ( struct STRXXSTARG *, const char *, const char *, unsigned long );
int TRXXSTargDisable ( struct STRXXSTARG *, const char *, const char *, unsigned long );
int TRXXSTargDup     ( struct STRXXSTARG *, struct STRXXSTARG *, unsigned long );
int TRXXSTargEnable  ( struct STRXXSTARG *, const char *, const char *, unsigned long );
int TRXXSTargExportgd( struct STRXXSTARG *, const char *, const char *, const char *, unsigned long );
int TRXXSTargFree    ( struct STRXXSTARG *, unsigned long );
int TRXXSTargInit    ( struct STRXXSTARG *, unsigned long );
int TRXXSTargInspect ( struct STRXXSTARG *, const char *, const char *, unsigned long );
int TRXXSTargIsValid ( struct STRXXSTARG *, unsigned long );
int TRXXSTargList    ( struct STRXXSTARG *, const char *, const char *, unsigned long );
int TRXXSTargMerge   ( struct STRXXSTARG *, struct STRXXSTARG *, unsigned long );
int TRXXSTargRead    ( struct STRXXSTARG *, const char *, unsigned long );
int TRXXSTargSpace   ( struct STRXXSTARG *, unsigned long );
int TRXXSTargWrite   ( struct STRXXSTARG *, const char *, const char *, const char *, unsigned long );

struct STRXXSTARG *_TRXXSTargTest( unsigned long );             

/*
   --- complete cross section data base 
*/
                                      /* */
struct STRXXSBASE {
   unsigned long ulSpace;             /* space counter */
                                      /* material list header */ 
   struct List sLH;         
   struct SCMDERR sCE;
};
/*
   Functions to handle cross section base
   --------------------------------------
   
   TRXXSBaseCalc:    ( xsbase, targbase, proj, targ, ia, flag );
                   7 x to test
		     o TargDup for partially mmap'ed data ?
		     o calls                 
		       o TRXTargDup()
                       o TRXTargBaseIsValid()
                       o TRXXSTargInit( &sXST, ulFlag );
                       o TRXXSTargFree( &sXST, ulFlag ); 
                       o TRXXSTargCalc( );
                       o TRXXSBaseTargAdd( );

   TRXXSBaseDelete:  ( xsbase, proj, targ, ia, flag);
                  10 o to complete
		     ! should not delete whole target entry
		     o calls
                       o TRXXSTargDelete( )
		       
   TRXXSBaseDisable: ( xsbase, proj, targ, ia, flag );
                 11a o to do
		     o wildcards for proj, ia
		     o calls
                       o TRXXSTargDisable( )
		        
   TRXXSBaseEnable:  ( xsbase, proj, targ, ia, flag );
                 11b o to do
		     o wildcards for proj, ia
		     o calls
                       o TRXXSTargEnable( )
		       
   TRXXSBaseExportgd:( xsbase, prefix, proj, targ, ia, flag );
                   8 x to test
		     o wildcards for proj, ia
		     o filenames as for *Write()
		     o calls
                       o TRXXSTargExportgd()
		       o TRXXSBaseIsValid()
		       
   TRXXSBaseFree:    ( xsbase, flag );
                   3 x to test
		     o calls
                       o TRXXSBaseInit()
		       o TRXXSTargFree()
		       
   TRXXSBaseInit:    ( xsbase, flag );
                   2 x to test
		     o calls

   TRXXSBaseInspect: ( xsbase, proj, targ, ia, flag );
                   1 x to test
		     o calls
		       o TRXXSTargInspect()

   TRXXSBaseIsValid: ( xsbase, flag );
                   4 o to do
		     o calls
                       o TRXXSTargIsValid()
		        
   TRXXSBaseList:    ( xsbase, proj, targ, ia, flag);
                  12 x to test
		     o better layout
		     o calls
                       o TRXXSTargList()
		       
   TRXXSBaseRead:    ( xsbase, prefix, proj, targ, ia, flag );
                     o to do
		     o inspect extension for valid ia
		     o proj, targ from contents
		     o calls                 
                       o TRXXSTargInit()
                       o TRXXSTargRead()
		       o TRXXSBaseTargAdd()
		       o TRXXSTargFree()

   TRXXSBaseSpace  : (xsbase, flag) count space
                   9 x to test
                     o calls
                    
   TRXXSBaseTargAdd: ( xsbase, xst, flag )
   		   6 x to test
		     ? how to treat mmap()ed stuff
		     ? compatibel w/ linked list, or array better
		     o calls
		       o TRXXSBaseTargLocate(  )
		       o TRXXSTargDup() 
		       o TRXXSTargFree()
		       o TRXXSTargInit()
		       o TRXXSTargMerge() 
		       
   TRXXSBaseTargLocate: ( xsbase, name )
                   5 x to test
                     o calls
			
   TRXXSBaseWrite: ( xsbase, prefix, proj, targ, ia, flag );
                     o todo
		     o consider TRXXS_SWAP
		     o calls
                       o TRXXSTargWrite()

		       
*/
                                      /* function prototypes */
int TRXXSBaseCalc(
                   struct STRXXSBASE *,
                   struct STRXTARGBASE *,
                   const char *,
                   const char *,
                   const char *,
                   struct STRXXSCALC *,
                   unsigned long
                 );
int TRXXSBaseDelete(
                     struct STRXXSBASE *,
                     const char *, const char *, const char *,
                     unsigned long
                   );
int TRXXSBaseDisable(
                     struct STRXXSBASE *,
                     const char *, const char *, const char *,
                     unsigned long
                   );
int TRXXSBaseEnable(
                     struct STRXXSBASE *,
                     const char *, const char *, const char *,
                     unsigned long
                   );
int TRXXSBaseExportgd(
                       struct STRXXSBASE *,
                       const char *,
                       const char *,
                       const char *,
                       const char *,
                       unsigned long
                     );
int TRXXSBaseFree   (
                     struct STRXXSBASE *,
                     unsigned long
                    );
int TRXXSBaseInit   (
                     struct STRXXSBASE *,
                     unsigned long
                    );
int TRXXSBaseInspect(
                     struct STRXXSBASE *,
                     const char *,
                     const char *,
                     const char *,
                     unsigned long
                    );
int TRXXSBaseIsValid(
                     struct STRXXSBASE *,
                     unsigned long
                    );
int TRXXSBaseList(
                     struct STRXXSBASE *,
                     const char *,
                     const char *,
                     const char *,
                     unsigned long
                    );
int TRXXSBaseSpace(
                     struct STRXXSBASE *,
                     unsigned long
                    );
int TRXXSBaseTargAdd( struct STRXXSBASE *, struct STRXXSTARG *, unsigned long );
struct STRXXSTARG *TRXXSBaseTargLocate( struct STRXXSBASE *, const char *, unsigned long );

/*
   --- error handling
*/

int TRXXSError( struct SCMDERR *, unsigned long );

enum { ERRTRXXS_SYS=1, 
       ERRTRXXS_BADADDR,
       ERRTRXXS_BADEDIM,
       ERRTRXXS_BADPROJ,
       ERRTRXXS_BADTARG,
       ERRTRXXS_BADE,
       ERRTRXXS_BADNSH,
       ERRTRXXS_BADSIGTOT,
       ERRTRXXS_BADWDIM,
       ERRTRXXS_BADADIM,
       ERRTRXXS_BADSIG2,
       ERRTRXXS_NOSUCHPROJ,
       ERRTRXXS_PROJDUPFAILED,
       ERRTRXXS_TARGDUPFAILED,
       ERRTRXXS_NOSUCHMETHOD,
       ERRTRXXS_BADW,
       ERRTRXXS_BADA,
       ERRTRXXS_BADINTERACTION,
       ERRTRXXS_BADXSPROJ,
       ERRTRXXS_BADXSTARG,
       ERRTRXXS_BADITEMLIST,
       ERRTRXXS_NOSUCHMATERIAL,
       ERRTRXXS_EOUTOFRANGE,
       ERRTRXXS_NOTCOMPTON,
       ERRTRXXS_BADCUMW,
       ERRTRXXS_BADCUMA,       
       ERRTRXXS_TARGMATCHFAILED,
       ERRTRXXS_PROJMATCHFAILED,
       ERRTRXXS_NOVALIDXS,
       ERRTRXXS_BASEEMPTY,
       ERRTRXXS_BADLIST,
       ERRTRXXS_RUDDUNSUPPTARG,
};

#endif
